/*
   code for rotative_plate + linear mvmt of pipette + servomotor for magnets + control Temp
*/

/*** init ***/

//libraries
#include <Arduino.h>
#include <Servo.h>
#include <DRV8825.h>
#include <Wire.h>
#include "fonctions.h"

//stepper motor
/*
    _R is for Rotative plate
    _L is for Linear mvmt (pipette)
*/

#define MOTOR_STEPS 200 //both stepper are Nema17

#define DIR_L 32
#define STEP_L 30
#define MODE2_L 28
#define MODE1_L 26
#define MODE0_L 24
#define ENABLE_L 22
#define SPEED_L 200
#define MICROSTEP_L 1

DRV8825 linearMvmtStep(MOTOR_STEPS, DIR_L, STEP_L, ENABLE_L, MODE0_L, MODE1_L, MODE2_L);

#define DIR_R 33
#define STEP_R 31
#define MODE2_R 29
#define MODE1_R 27
#define MODE0_R 25
#define ENABLE_R 23
#define SPEED_R 1
#define MICROSTEP_R 32

DRV8825 rotativePlateStep(MOTOR_STEPS, DIR_R, STEP_R, ENABLE_R, MODE0_R, MODE1_R, MODE2_R);

//servomotor
Servo magnetServo;  // create servo object to control a servo
#define MAGNET_ON 30
#define MAGNET_OFF 160
#define MAGNET_SERVO_PIN 7

//Pipette PIN attributions
#define Down_PIN 51
#define Up_PIN 47
#define Select_PIN 43
#define Enter_PIN  45 
#define EjectTip_PIN 49
#define Center_PIN 41

//switch
#define ROTATIVE_PLATE_SWITCH_PELTIER A4
#define ROTATIVE_PLATE_SWITCH_TRANSFO A1
#define ROTATIVE_PLATE_SWITCH_AMPLI A0
#define PIPETTE_SWITCH A7
//int pos0_L;
//int pos0_R;

//actuator
Servo myServo;
//#define PIN_SERVO 8




//Temperature Monitoring
#define heaterAmpli_Pin 7// Commande cartouche chauffante pour amplification
#define heaterPin 8 //Commande de la cartouche chauffante transfo heat
#define coolerPin 3 // Commande du Peltier
#define DELAY1 100 //temps d attente lors de la mesure
#define hysteresis 1
int cycles = 50; //Number of cycles for avg of temperature
#define I2C_ADDR 0x5A // Adresse par dÃ©faut du capteur IR
#define PIN_SERVO 11


// PURIFICATION parameters
#define V_0 1000
#define V_1_uL 100
#define V_9_uL 900
#define ANGLE_pos0 50
#define ANGLE_1_uL -104
#define ANGLE_9_uL -108.5
#define ANGLE_BETWEEN_2_HOLES 4.5
#define HEIGHT 500
#define ANGLE_PELTIER_TO_IR 51.7
#define ANGLE_PELTIER_TO_TIP -26.1
#define ANGLE_PELTIER_TO_BIG_BUFFER -110
#define ANGLE_BIG_BUFFER_TO_BUFFER -20
#define ANGLE_BTW_TIPS -4.55

#define ANGLE_AMPLI_TO_BUFFER 16

#define COOLING_TEMP 11.5
#define HEIGHT_BOTTOM_BUFFER 1800
#define HEIGHT_BOTTOM_TUBE 1650
#define ANGLE_BTW_TUBES -4.35
#define ANGLE_TIP_TO_BUFFER 54.8
#define ANGLE_BUFFER_TO_1_TUBE_PELTIER -12

#define ANGLE_TIP_TO_MAGBEADS 114.5
#define ANGLE_TIP_TO_BUFFER1 120.2
#define ANGLE_TIP_TO_BUFFER2 125.8
#define ANGLE_TIP_TO_BUFFER3 131
#define ANGLE_TIP_TO_BACT 68.1

#define ANGLE_MAGBEADS_TO_PURIF 49
#define ANGLE_BUFFER1_TO_PURIF 43.3
#define ANGLE_BUFFER2_TO_PURIF 37.9
#define ANGLE_BUFFER3_TO_PURIF 32.5

#define ANGLE_BACT_TO_PURIF 95
#define ANGLE_PURIF_TO_BUFFER1 -43.3
#define ANGLE_PURIF_TO_BUFFER2 -37.9
#define ANGLE_PURIF_TO_BUFFER3 -32.5
#define ANGLE_PURIF_TO_WASTE -27.1
#define ANGLE_PURIF_TO_STORE -21.7
#define height_buffer 2200
#define height_purif 1900
#define height_bacteria 2000

#define DELAY_SECURE 1500

void setup() {
  //  int Communication
  Serial.begin( 9600 );
  Serial.println("boot");
  // heat & cooling pin
  pinMode(heaterAmpli_Pin, OUTPUT);
  pinMode(heaterPin, OUTPUT);
  pinMode(coolerPin, OUTPUT);
  pinMode(ROTATIVE_PLATE_SWITCH_TRANSFO , INPUT);
  pinMode(PIPETTE_SWITCH , INPUT);
  pinMode(ROTATIVE_PLATE_SWITCH_AMPLI , INPUT);
  pinMode(ROTATIVE_PLATE_SWITCH_PELTIER , INPUT);

  // stepper motors init
  linearMvmtStep.begin(SPEED_L, MICROSTEP_L);
  linearMvmtStep.disable();
  rotativePlateStep.begin(SPEED_R, MICROSTEP_R);
  rotativePlateStep.disable();
  int stepsL = 200;
  int stepsR = 75;
  Serial.println("end boot");

  //!!!!!!!!!!!!!!!!!!INIT!!!!!!!!!!!!!!!!!!
  Serial.println("start init & calibration");
  digitalWrite(coolerPin, LOW);
  digitalWrite(heaterPin, LOW);
  digitalWrite(heaterAmpli_Pin, LOW);
  myServo.attach(PIN_SERVO);


  // STEP 1 : ADD 200 uL of LYSIS BUFFER
  initPipette();
  linearPipetteSetup(stepsL);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  delay(2000);
  takeTip();
  rotatePlate(ANGLE_TIP_TO_BUFFER1);
  delay(2000);
  setvolume(200,300);
  goAspirate(height_buffer);
  rotatePlate(ANGLE_BUFFER1_TO_PURIF);
  delay(2000);
  goDispense(height_purif);

  
  //  STEP 2 : ADD 100 uL of BACTERIA + wait 10 minutes
  rotatePlate(-ANGLE_BUFFER1_TO_PURIF-ANGLE_TIP_TO_BUFFER1);
  delay(2000);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  releaseTip();
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS);
  takeTip();
  delay(2000);
  rotatePlate(-ANGLE_BTW_TIPS);
  delay(2000);
  rotatePlate(ANGLE_TIP_TO_BACT);
  delay(2000);
  setvolume(100,200);
  goAspirate(height_bacteria);
  rotatePlate(ANGLE_BACT_TO_PURIF);
  delay(2000);
  goDispense(height_purif);
  delay(600000); // 10 minutes of incubation to let the ADN-ARN go in the solution
  triggerButton(Enter_PIN); // Wake up the pipette


  //  STEP 3 : ADD 20uL of MAGBEADS + wait 10 minutes
  rotatePlate(-ANGLE_BUFFER1_TO_PURIF-ANGLE_TIP_TO_BUFFER1);
  delay(2000);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS);
  delay(2000);
  releaseTip();
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS); 
  delay(2000);
  takeTip();
  delay(2000);
  rotatePlate(-2*ANGLE_BTW_TIPS+0.3); 
  delay(2000);
  rotatePlate(ANGLE_TIP_TO_MAGBEADS);
  delay(2000);
  setvolume(20,100);
  goAspirate(height_buffer);
  rotatePlate(ANGLE_MAGBEADS_TO_PURIF); 
  delay(2000);
  goDispense(height_purif);
  delay(600000); // 10 minutes of incubation 
  triggerButton(Enter_PIN); // Wake up the pipette

  
  // STEP 4 : Bring MAGNETS next to the tube to attract the magnetic beads and EXTRACT the WASTE liquid
  rotatePlate(-ANGLE_BUFFER1_TO_PURIF-ANGLE_TIP_TO_BUFFER1);
  delay(2000);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  delay(2000);
  rotatePlate(2*ANGLE_BTW_TIPS);
  releaseTip();
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS-0.2);
  takeTip();
  delay(2000);
  rotatePlate(-3*ANGLE_BTW_TIPS+0.4);
  delay(2000);
  rotatePlate(ANGLE_TIP_TO_MAGBEADS);
  delay(2000);
  rotatePlate(ANGLE_MAGBEADS_TO_PURIF);
  delay(2000);
  // ON RAPPROCHE L'AIMANT
  delay(5000);
  myServo.attach(PIN_SERVO);
  delay(5000);
  SetStrokePerc(94);
  delay(10000);
  setvolume(300,300);
  goAspirate(height_purif);
  SetStrokePerc(1);
  delay(7000);
  //GO TO THE WASTE
  rotatePlate(ANGLE_PURIF_TO_WASTE-0.3);
  delay(2000);
  goDispense(height_buffer);
  
  
  // STEP 5 : Add 300µL of BUFFER 1 (Lysis Buffer), FLUSH, EXTRACT the liquid while magnets attract the beads and throw to the WASTE.
  rotatePlate(-ANGLE_BUFFER1_TO_PURIF-ANGLE_TIP_TO_BUFFER1-ANGLE_PURIF_TO_WASTE);
  delay(2000);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  delay(2000);
  rotatePlate(3*ANGLE_BTW_TIPS);
  releaseTip();
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS-0.5);
  takeTip();
  delay(2000);
  rotatePlate(-4*ANGLE_BTW_TIPS+0.5);
  //CHANGE THE MODE OF THE PIPETTE TO P+MIXING FOR THE FLUSHING
  mode_P_to_flush();
  setvolume_flush(300,300);
  // MOVE TO BUFFER 1
  rotatePlate(ANGLE_TIP_TO_BUFFER1);
  delay(2000);
  goAspirate(height_buffer);
  rotatePlate(ANGLE_BUFFER1_TO_PURIF);
  delay(2000);
  linearBottom(height_purif);
  delay(2000);
  dispense();
  delay(4000);
  //FLUSHING
  triggerButton_Delay(Center_PIN, 8000);
  delay(3000);
  linearTop(1200); 
  triggerButton_Delay(Center_PIN, 1000);
  delay(1000);
  triggerButton_Delay(Center_PIN, 1000);
  delay(3000);
  linearTop(height_purif-1200); 
  delay(3000);
  // CHANGE THE MODE BACK TO PIPETTE
  mode_flush_to_P();
  setvolume(300,300);
  triggerButton(Enter_PIN);
  delay(2000);
  // BRING MAGNETS NEXT TO THE TUBE
  delay(5000);
  SetStrokePerc(94);
  delay(10000);
  setvolume(300,300);
  goAspirate(height_purif);
  SetStrokePerc(1);
  delay(7000);
  //GO TO THE WASTE
  rotatePlate(ANGLE_PURIF_TO_WASTE-0.3);
  delay(2000);
  goDispense(height_buffer);


  // STEP 6 : (Add 300µL of BUFFER 2 (Wash 2), EXTRACT while magnets attract the beads and throw to the WASTE the liquid) * 2 times
  rotatePlate(-ANGLE_BUFFER1_TO_PURIF-ANGLE_TIP_TO_BUFFER1-ANGLE_PURIF_TO_WASTE);
  delay(2000);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  delay(2000);
  rotatePlate(4*ANGLE_BTW_TIPS-0.2);
  releaseTip();
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS-0.7);
  takeTip();
  delay(2000);
  rotatePlate(-5*ANGLE_BTW_TIPS+0.6);
  delay(2000);
  rotatePlate(ANGLE_TIP_TO_BUFFER2);
  delay(2000);
  setvolume(300,300);
  goAspirate(height_buffer);
  rotatePlate(ANGLE_BUFFER2_TO_PURIF);
  delay(2000);
  goDispense(height_purif);
  delay(5000);
  SetStrokePerc(94);
  delay(10000);
  setvolume(300,300);
  goAspirate(height_purif);
  SetStrokePerc(1);
  delay(7000);
  //GO TO WASTE
  rotatePlate(ANGLE_PURIF_TO_WASTE-0.3);
  delay(2000);
  goDispense(height_buffer);
  // GO TO BUFFER 2
  rotatePlate(ANGLE_PURIF_TO_BUFFER2-0.9-ANGLE_PURIF_TO_WASTE);
  delay(2000);
  setvolume(300,300);
  goAspirate(height_buffer);  
  // GO TO PURIF
  rotatePlate(-ANGLE_PURIF_TO_WASTE+0.3-(ANGLE_PURIF_TO_BUFFER2-0.9-ANGLE_PURIF_TO_WASTE));
  delay(2000);
  goDispense(height_purif);
  delay(5000);
  SetStrokePerc(94);
  delay(10000);
  setvolume(300,300);
  goAspirate(height_purif);
  SetStrokePerc(1);
  delay(7000);
  //GO TO WASTE
  rotatePlate(ANGLE_PURIF_TO_WASTE-0.3);
  delay(2000);
  goDispense(height_buffer);
  
  

    // STEP 7 : Add 300µL of BUFFER 3 (Elution), EXTRACT while magnets attract the beads and throw to the WASTE the liquid.

  rotatePlate(-ANGLE_BUFFER1_TO_PURIF-ANGLE_TIP_TO_BUFFER1-ANGLE_PURIF_TO_WASTE);
  delay(2000);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  delay(2000);
  rotatePlate(5*ANGLE_BTW_TIPS-0.7);
  releaseTip();
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS-0.2);
  takeTip();
  delay(2000);
  rotatePlate(-6*ANGLE_BTW_TIPS+1);
  delay(2000);
  rotatePlate(ANGLE_TIP_TO_BUFFER3); //-0.8);
  delay(2000);
  setvolume(300,300);
  goAspirate(height_buffer);
  rotatePlate(ANGLE_BUFFER3_TO_PURIF); //+0.8);
  delay(2000);
  goDispense(height_purif);
  delay(5000);
  SetStrokePerc(94);
  delay(10000);
  //setvolume(300,300);
  goAspirate(height_purif);
  SetStrokePerc(1);
  delay(7000);
  //GO TO WASTE
  rotatePlate(ANGLE_PURIF_TO_WASTE-0.3);
  delay(2000);
  goDispense(height_buffer);


  // STEP 8 : ADD 30µL of BUFFER 3 (Elution) + heat to 70°C during 5 minutes + Flush
  rotatePlate(-ANGLE_BUFFER1_TO_PURIF-ANGLE_TIP_TO_BUFFER1-ANGLE_PURIF_TO_WASTE);
  delay(2000);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  delay(2000);
  rotatePlate(6*ANGLE_BTW_TIPS-1);
  releaseTip();
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS-0.7);
  takeTip();
  delay(2000);
  rotatePlate(-7*ANGLE_BTW_TIPS+1.5);
  delay(2000);
  rotatePlate(ANGLE_TIP_TO_BUFFER3);
  delay(2000);
  setvolume(30,300);
  goAspirate(height_buffer);
  rotatePlate(ANGLE_BUFFER3_TO_PURIF);
  delay(2000);
  goDispense(height_purif);
  delay(5000);
  rotatePlate(ANGLE_PELTIER_TO_IR);
  delay(10000);
  float temp=tempAvg();
  heatAmplification(70);
  rotatePlate(-ANGLE_PELTIER_TO_IR-0.5);

  //CHANGE THE MODE Of THE PIPETTE TO P+MIXING FOR THE FLUSHING
  triggerButton(Enter_PIN); // Wake up the pipette
  mode_P_to_flush();
  setvolume_flush(30,300);
  delay(2000);
  linearBottom(height_purif);
  aspirate();
  delay(4000);
  dispense();
  // FLUSHING
  triggerButton(Center_PIN);
  triggerButton(Center_PIN);
  delay(2000);
  triggerButton_Delay(Center_PIN, 8000);
  delay(3000);                       
  linearTop(750); 
  triggerButton_Delay(Center_PIN, 1000);
  delay(1000);
  triggerButton_Delay(Center_PIN, 1000);
  delay(10000);
  linearTop(height_purif-750); 
  delay(1000);

  rotatePlate(ANGLE_PELTIER_TO_IR);
  delay(2000);
  temp=tempAvg();
  heatAmplification(70);

  rotatePlate(-ANGLE_PELTIER_TO_IR-0.5);
  delay(2000);
  linearBottom(height_purif);
  aspirate();
  delay(4000);
  dispense();
  //FLUSHING
  triggerButton(Center_PIN);
  triggerButton(Center_PIN);
  delay(2000);
  triggerButton_Delay(Center_PIN, 8000);
  delay(3000);                        
  linearTop(750); 
  triggerButton_Delay(Center_PIN, 1000);
  delay(1000);
  triggerButton_Delay(Center_PIN, 1000);
  delay(10000);
  linearTop(height_purif-750); 
  delay(1000);
  // CHANGE THE MODE BACK TO PIPETTE
  mode_flush_to_P();
  delay(2000);


  // STEP 9 : EXTRACT while magnets attract the beads and store the liquid (DNA)
  rotatePlate(-ANGLE_BUFFER1_TO_PURIF-ANGLE_TIP_TO_BUFFER1-ANGLE_PURIF_TO_WASTE);
  delay(2000);
  rotativePlateSetup_Peltier(stepsR);
  delay(2000);
  rotatePlate(ANGLE_PELTIER_TO_TIP);
  delay(2000);
  rotatePlate(7*ANGLE_BTW_TIPS-0.8);
  releaseTip();
  delay(2000);
  rotatePlate(ANGLE_BTW_TIPS-0.5);
  takeTip();
  delay(2000);
  rotatePlate(-8*ANGLE_BTW_TIPS+1);
  delay(2000);
  rotatePlate(ANGLE_TIP_TO_BUFFER3);
  delay(2000);
  rotatePlate(ANGLE_BUFFER3_TO_PURIF);
  delay(5000);
  SetStrokePerc(94);
  delay(10000);
  setvolume(30,30);
  goAspirate(height_purif);
  SetStrokePerc(1);
  delay(7000);
  // GO TO STORE DNA
  rotatePlate(ANGLE_PURIF_TO_STORE);
  delay(2000);
  goDispense(height_buffer);
}
  
  

void loop() {

}





