#include <Arduino.h>
#include <Servo.h>
#include <DRV8825.h>
#include <Wire.h>

extern Servo magnetServo, myServo;

#define MOTOR_STEPS 200 //both stepper are Nema17

#define DIR_L 32
#define STEP_L 30
#define MODE2_L 28
#define MODE1_L 26
#define MODE0_L 24
#define ENABLE_L 22
#define SPEED_L 200
#define MICROSTEP_L 1

extern DRV8825 linearMvmtStep;

#define DIR_R 33
#define STEP_R 31
#define MODE2_R 29
#define MODE1_R 27
#define MODE0_R 25
#define ENABLE_R 23
#define SPEED_R 1
#define MICROSTEP_R 32

extern DRV8825 rotativePlateStep;

#define MAGNET_ON 30
#define MAGNET_OFF 160
#define MAGNET_SERVO_PIN 7

// pin definitions for pipette
//#define Down_PIN 51
//#define Up_PIN 49
//#define Select_PIN 47
//#define Enter_PIN  45
//#define EjectTip_PIN 43
//#define Center_PIN 41

#define Down_PIN 51
#define Up_PIN 47
#define Select_PIN 43
#define Enter_PIN  45
#define EjectTip_PIN 49
#define Center_PIN 41

//switch
#define ROTATIVE_PLATE_SWITCH_PELTIER A4
#define ROTATIVE_PLATE_SWITCH_TRANSFO A1
#define ROTATIVE_PLATE_SWITCH_AMPLI A0
#define PIPETTE_SWITCH A7
//int pos0_L;
//int pos0_R;

//actuator
// #define PIN_SERVO 8


//Temperature Monitoring
#define heaterAmpli_Pin 7// Commande cartouche chauffante pour amplification
#define heaterPin 8 //Commande de la cartouche chauffante pour transfo heat
#define coolerPin 3 // Commande du Peltier

#define DELAY1 100 //temps d attente lors de la mesure
#define hysteresis 1
extern int cycles; //Number of cycles for avg of temperature
#define I2C_ADDR 0x5A // Adresse par dÃ©faut du capteur IR




//linear movement of pipette :

void linearTop(int height)
{
  //move pipette above tube :

  linearMvmtStep.enable();
  linearMvmtStep.move(height);
  linearMvmtStep.disable();
  delay(300);
}

void linearBottom(int height)
{
  //move pipette to bottom of tube :

  linearMvmtStep.enable();
  linearMvmtStep.move(-height);
  linearMvmtStep.disable();
  delay(300);
}


int linearPipetteSetup(int steps)
{
  int threshold = 400;
  int value = 0;
  int countStep = 0;
  int flagSwitch = 0;
  int startSwitch = -1;
  int preciseStep = 10;

  //linearBottom(400);

  while (startSwitch == -1)
  {
    //recherche switch
    linearBottom(steps);
    countStep += steps;
    value = analogRead(PIPETTE_SWITCH);
    //Serial.println(value);
    //    Serial.println(",");
    //Serial.println(countStep);

    
    //recherche dï¿½but plage
    if ((value > threshold) && (flagSwitch == 0))
    {
      flagSwitch = 1;
      //linearBottom(450);
      while (value > threshold)
      {
        value = analogRead(PIPETTE_SWITCH);
        //Serial.println(value);
        linearBottom(preciseStep);
        countStep -= preciseStep;
        //Serial.println(countStep);
      }

      startSwitch = countStep;
      //      Serial.print("startSwitch = ");
      //      Serial.println(startSwitch);
    }

  }
  linearTop(2100);
  return (startSwitch);
}

//rotation of plate :
void rotatePlate(float angle)
{
  //rotate plate from angle
  //delay(200);
  rotativePlateStep.enable();
  rotativePlateStep.rotate(angle);
  rotativePlateStep.disable();
  delay(200);
}

void movePlate(int steps)
{
  //move plate from steps

  rotativePlateStep.enable();
  rotativePlateStep.move(steps);
  rotativePlateStep.disable();
  delay(200);

}
void moveToStep(int currentStep, int goalStep)
{
  delay(500);
  int steps = goalStep - currentStep;
  rotativePlateStep.enable();
  rotativePlateStep.move(steps);
  rotativePlateStep.disable();
  delay(500);
}

int rotativePlateSetup_Peltier(int steps)
{
  int rotative0;
  int threshold = 400;
  int value = 0;
  int countStep = 0;
  int flagSwitch = 0;
  int endSwitch = -1;
  int startSwitch = -1;

  while (endSwitch == -1)
  {
    movePlate(steps);
    countStep += steps;
    value = analogRead(ROTATIVE_PLATE_SWITCH_PELTIER);
    //Serial.println(value);
    //Serial.println("HERE");
    //    Serial.println(countStep);

    if ((value > threshold) && (flagSwitch == 0))
    {
      flagSwitch = 1;

      while (analogRead(ROTATIVE_PLATE_SWITCH_PELTIER) > threshold)
      {
        //Serial.println(analogRead(ROTATIVE_PLATE_SWITCH_PELTIER));
        movePlate(-steps / 16);
        //Serial.println("small steps");
        countStep -= steps / 16;
        delay(200);
        //        Serial.println(countStep);
      }

      startSwitch = countStep;
      //      Serial.print("startSwitch = ");
      //      Serial.println(startSwitch);

      movePlate(2*steps);
      countStep += 2*steps;
      //      Serial.println(countStep);

      while (analogRead(ROTATIVE_PLATE_SWITCH_PELTIER) > threshold)
      {
        movePlate(steps / 16);
        countStep += steps / 16;
        delay(200);
        //        Serial.println(countStep);
      }

      endSwitch = countStep;
      //      Serial.print("endSwitch = ");
      //      Serial.println(endSwitch);
    }

    delay(500);
  }
  rotative0 = (endSwitch + startSwitch) / 2;
  //  Serial.print("rotative0 = ");
  //  Serial.println(rotative0);
  moveToStep(countStep, rotative0);


  delay(500);
  return rotative0;
}


int rotativePlateSetup_HeatingTransfo(int steps)
{
  int rotative0;
  int threshold = 400;
  int value = 0;
  int countStep = 0;
  int flagSwitch = 0;
  int endSwitch = -1;
  int startSwitch = -1;

  while (endSwitch == -1)
  {
    movePlate(steps);
    countStep += steps;
    value = analogRead(ROTATIVE_PLATE_SWITCH_TRANSFO);
    //Serial.println(value);
    //Serial.println("HERE");
    //    Serial.println(countStep);

    if ((value > threshold) && (flagSwitch == 0))
    {
      flagSwitch = 1;

      while (analogRead(ROTATIVE_PLATE_SWITCH_TRANSFO) > threshold)
      {
        //Serial.println(analogRead(ROTATIVE_PLATE_SWITCH_TRANSFO));
        movePlate(-steps / 16);
        //Serial.println("small steps");
        countStep -= steps / 16;
        delay(200);
        //        Serial.println(countStep);
      }

      startSwitch = countStep;
      //      Serial.print("startSwitch = ");
      //      Serial.println(startSwitch);

      movePlate(3*steps);
      countStep += 3*steps;
      //      Serial.println(countStep);

      while (analogRead(ROTATIVE_PLATE_SWITCH_TRANSFO) > threshold)
      {
        movePlate(steps / 16);
        countStep += steps / 16;
        delay(200);
        //        Serial.println(countStep);
      }

      endSwitch = countStep;
      //      Serial.print("endSwitch = ");
      //      Serial.println(endSwitch);
    }

    delay(500);
  }
  rotative0 = (endSwitch + startSwitch) / 2;
  //  Serial.print("rotative0 = ");
  //  Serial.println(rotative0);
  moveToStep(countStep, rotative0);


  delay(500);
  return rotative0;
}


int rotativePlateSetup_Ampli(int steps)
{
  int rotative0;
  int threshold = 400;
  int value = 0;
  int countStep = 0;
  int flagSwitch = 0;
  int endSwitch = -1;
  int startSwitch = -1;

  while (endSwitch == -1)
  {
    movePlate(steps);
    countStep += steps;
    value = analogRead(ROTATIVE_PLATE_SWITCH_AMPLI);
    //Serial.println(value);
    //Serial.println("HERE");
    //    Serial.println(countStep);

    if ((value > threshold) && (flagSwitch == 0))
    {
      flagSwitch = 1;

      while (analogRead(ROTATIVE_PLATE_SWITCH_AMPLI) > threshold)
      {
        //Serial.println(analogRead(ROTATIVE_PLATE_SWITCH_AMPLI));
        movePlate(-steps / 16);
        //Serial.println("small steps");
        countStep -= steps / 16;
        delay(200);
        //        Serial.println(countStep);
      }

      startSwitch = countStep;
      //      Serial.print("startSwitch = ");
      //      Serial.println(startSwitch);

      movePlate(3*steps);
      countStep += 3*steps;
      //      Serial.println(countStep);

      while (analogRead(ROTATIVE_PLATE_SWITCH_AMPLI) > threshold)
      {
        movePlate(steps / 16);
        countStep += steps / 16;
        delay(200);
        //        Serial.println(countStep);
      }

      endSwitch = countStep;
      //      Serial.print("endSwitch = ");
      //      Serial.println(endSwitch);
    }

    delay(500);
  }
  rotative0 = (endSwitch + startSwitch) / 2;
  //  Serial.print("rotative0 = ");
  //  Serial.println(rotative0);
  moveToStep(countStep, rotative0);


  delay(500);
  return rotative0;
}

//magnet control

void ActivateMagnet()
{
  //bring magnets close to the tubes
  int pos;
  for (pos = MAGNET_OFF; pos >= MAGNET_ON; pos -= 1)
  { // goes from 180 degrees to 0 degrees
    magnetServo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(15);                       // waits 15ms for the servo to reach the position
  }
}

void DeactivateMagnet()
{
  //bring magnets far away from the tubes
  int pos;
  for (pos = MAGNET_ON; pos <= MAGNET_OFF; pos += 1)
  { // goes from 180 degrees to 0 degrees
    magnetServo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(15);                       // waits 15ms for the servo to reach the position
  }
}


//pipette control

void triggerButton( int pin )
{

  // trigger a button using a transistor
  digitalWrite( pin, HIGH );
  delay(175);
  digitalWrite( pin, LOW );
  delay(175);
}


void triggerButton_Delay( int pin , unsigned long time_delay)
{
  digitalWrite( pin, HIGH );
  delay(time_delay);
  digitalWrite( pin, LOW );
  delay(175);
}

void initPipette()
{
  // initialize pipette : pin and start of pipette
  pinMode( Enter_PIN, OUTPUT );
  digitalWrite( Enter_PIN, LOW );

  pinMode( EjectTip_PIN, OUTPUT );
  digitalWrite( EjectTip_PIN, LOW );

  pinMode( Center_PIN, OUTPUT );
  digitalWrite( Center_PIN, LOW );

  pinMode( Down_PIN, OUTPUT );
  digitalWrite( Down_PIN, LOW );

  pinMode( Up_PIN, OUTPUT );
  digitalWrite( Up_PIN, LOW );

  pinMode( Select_PIN, OUTPUT );
  digitalWrite( Select_PIN, LOW );

  // INITIALISATION
  triggerButton(EjectTip_PIN);
  Serial.println("init");
  delay(8000);
  //Serial.println("action");
}


void mode_P_to_flush()
{
  triggerButton(Select_PIN);
  delay(500);
  triggerButton(Up_PIN);
  triggerButton(Up_PIN);
  delay(500);
  triggerButton(Enter_PIN);
  delay(500);
}

void mode_flush_to_P()
{
  triggerButton(Select_PIN);
  delay(500);
  triggerButton(Down_PIN);
  triggerButton(Down_PIN);
  delay(500);
  triggerButton(Enter_PIN);
  delay(500);
}

void aspirate()
{
  delay(1000);
  triggerButton(Center_PIN);
}

void goAspirate(int height)
{
  //go down, aspirate, go back up

  linearBottom(height);
  aspirate();
  delay(4000);
  linearTop(height);
  delay(1000);
}

void dispense()
{
  delay(1000);
  triggerButton(Center_PIN);
}

void goDispense(int height)
{
  //go down, dispense, go back up
  int height_security=1100;
  linearBottom(height-height_security);
  dispense();
  delay(5000);
  linearTop(height-height_security);
  delay(1000);
}

void flushing() // Think to switch the mode to Pipette+Mix
{

  delay(2000);
  triggerButton_Delay(Center_PIN, 8000);
  delay(3000);
  triggerButton_Delay(Center_PIN, 1000);
  delay(1000);
  triggerButton_Delay(Center_PIN, 1000);
}
void takeTip()
{
  int height1 =2400;//real value : 1600   1000
  int height2 =650;
//  int height3 = 300;
  delay(1000);
  linearBottom(height1);
  delay(1000);
  linearBottom(height2);
  linearTop(height1+height2);
}
void releaseTip()
{
  int height1 =2000; //real value 1750
  linearBottom(height1);
  delay(1000);
  int height2 =600;
  linearBottom(height2);
  delay(500);
  triggerButton(EjectTip_PIN);
  delay(300);
  triggerButton(EjectTip_PIN);
  delay(300);
  triggerButton(EjectTip_PIN);
  delay(1000);
  linearTop(height1+height2);
}
unsigned long compute_delay_flush(int delta_V)
{
  //compute the delay in order to set the volume
  //delta_V is in uL
  unsigned long delay_time = int(((float(delta_V) +40.2857142857)/22.709091)*1000);
  return delay_time;
}

unsigned long compute_delay(int delta_V)
{
  //compute the delay in order to set the volume
  //delta_V is in uL
  unsigned long delay_time = int(((float(delta_V) +32.2857142857)/22.709091)*1000);
  return delay_time;
}

void setvolume_flush(int volume_end, int volume_start)
{
  //set the volume (in microliters)

  int delta_V = abs(volume_end - volume_start);
  Serial.println(delta_V);
  // Si on veut changer le volume de plus de 1uL on reste appuyï¿½ sur le bouton un certain temps
  if (delta_V >= 10)
  {
    unsigned long time_delay = compute_delay_flush(delta_V);
    Serial.println(time_delay);
    delay(1000);
    Serial.println("time_delay");
    
    if (volume_end > volume_start)
    {
      triggerButton(Up_PIN);
      triggerButton_Delay( Up_PIN , time_delay);
    }
    else if (volume_end < volume_start)
    {
      triggerButton(Down_PIN);
      triggerButton_Delay( Down_PIN , time_delay);
    }
  }

  ////Si on veut changer le volume de moins de 1uL on va de 0.05uL en 0.05uL en appuyant succesivement sur le bouton
  else if (delta_V < 10)
  {
    delta_V;
    if (volume_end > volume_start)
    {
      triggerButton(Up_PIN);
      for (int t = 0; t < delta_V ; t = t + 1)
      {
        triggerButton(Up_PIN);
      }
    }
    else if (volume_end < volume_start)
    {
      triggerButton(Down_PIN);
      for (int t = 0; t < delta_V ; t = t + 1)
      {
        triggerButton(Down_PIN);
      }
    }
  }
  delay(1000);
  triggerButton(Enter_PIN);
  delay(1000);
}

void setvolume(int volume_end, int volume_start)
{
  //set the volume (in microliters)

  int delta_V = abs(volume_end - volume_start);
  Serial.println(delta_V);
  // Si on veut changer le volume de plus de 1uL on reste appuyï¿½ sur le bouton un certain temps
  if (delta_V >= 10)
  {
    unsigned long time_delay = compute_delay(delta_V);
    Serial.println(time_delay);
    delay(1000);
    Serial.println("time_delay");
    
    if (volume_end > volume_start)
    {
      triggerButton(Up_PIN);
      triggerButton_Delay( Up_PIN , time_delay);
    }
    else if (volume_end < volume_start)
    {
      triggerButton(Down_PIN);
      triggerButton_Delay( Down_PIN , time_delay);
    }
  }

  ////Si on veut changer le volume de moins de 1uL on va de 0.05uL en 0.05uL en appuyant succesivement sur le bouton
  else if (delta_V < 10)
  {
    delta_V;
    if (volume_end > volume_start)
    {
      triggerButton(Up_PIN);
      for (int t = 0; t < delta_V ; t = t + 1)
      {
        triggerButton(Up_PIN);
      }
    }
    else if (volume_end < volume_start)
    {
      triggerButton(Down_PIN);
      for (int t = 0; t < delta_V ; t = t + 1)
      {
        triggerButton(Down_PIN);
      }
    }
  }
  delay(1000);
  triggerButton(Enter_PIN);
  delay(1000);
}


void triggerButton_Long( int pin )
{
  digitalWrite( pin, HIGH );
  delay(2299);
  digitalWrite( pin, LOW );
  delay(175);
}


void setspeed(int speed_start_in, int speed_end_in, int speed_start_out, int speed_end_out)
{
  // Set the Speeds IN/OUT from 1 to 9
  // CHANGE THE SPEED IN
  int delta_speed_in = speed_end_in - speed_start_in;
  if (delta_speed_in > 0)
  {
    for (int t = 0; t < delta_speed_in ; t = t + 1)
    {
      triggerButton(Up_PIN);
    }
  }
  else if (delta_speed_in < 0)
  {
    for (int t = 0; t < (-delta_speed_in) ; t = t + 1)
    {
      triggerButton(Down_PIN);
    }
  }
  //VALIDATE
  triggerButton(Enter_PIN);
  delay(1000);

  // CHANGE THE SPEED OUT
  int delta_speed_out = speed_end_out - speed_start_out;

  if (delta_speed_out > 0)
  {
    for (int t = 0; t < delta_speed_out ; t = t + 1)
    {
      triggerButton(Up_PIN);
    }
  }
  else if (delta_speed_out < 0)
  {
    for (int t = 0; t < (-delta_speed_out) ; t = t + 1)
    {
      triggerButton(Down_PIN);
    }
  }

  //VALIDATE
  delay(175);
  triggerButton(Enter_PIN);
}

//Temp Monitoring

float tempAvg()
{
  Wire.begin();
  //measure average temperature over #cycle
  float resultTemp = 0.0;
  for (int i = 0; i < cycles; i++) {
    /* Donnï¿½es brute de tempï¿½rature */
    uint16_t data;

    /* Commande de lecture de la RAM ï¿½ l'adresse 0x07 */
    Wire.beginTransmission(I2C_ADDR);
    Wire.write(0x07);
    Wire.endTransmission(false);

    /* Lecture des donnï¿½es : 1 mot sur 16 bits + octet de contrï¿½le (PEC) */
    Wire.requestFrom(I2C_ADDR, 3, false);
    while (Wire.available() < 3);
    data = Wire.read();
    data |= (Wire.read() & 0x7F) << 8;  // Le MSB est ignorï¿½ (bit de contrï¿½le d'erreur)
    Wire.read(); // PEC
    Wire.endTransmission();

    /* Calcul de la tempï¿½rature */
    const float tempFactor = 0.02; // 0.02ï¿½C par LSB -> rï¿½solution du MLX90614
    float tempData = (tempFactor * data) - 0.01;
    float celsius = tempData - 273.15; // Conversion des degrï¿½s Kelvin en degrï¿½s Celsius



    /* Temps d'attente */
    resultTemp += celsius; // Addition des valeurs mesurees
    delay(DELAY1);
  }
  resultTemp /= cycles;  // calcul de la moyenne
  return (resultTemp);
}

int tempMonitoringHeatAmpli(float resultTemp, float wantedTemp)
{
  //control heating device accordingly to the measured temp
  //return 1 if higher than wanted Temp+hyst, 0 if wantedTemp, -1 lower
  resultTemp = tempAvg();

  /* Affichage de la tempï¿½rature */
  Serial.print(resultTemp);
  Serial.println("Celsius: ");
  // switch de la rï¿½sistance

  if (resultTemp < wantedTemp - hysteresis)
  { digitalWrite(heaterAmpli_Pin, HIGH);
    //Serial.println("HIGH");
    return (-1);
  }
  else if (resultTemp > wantedTemp + hysteresis)
  { digitalWrite(heaterAmpli_Pin, LOW);
    //Serial.println("LOW");
    return (1);
  }
}
  
int tempMonitoringHeat(float resultTemp, float wantedTemp)
{
  //control heating device accordingly to the measured temp
  //return 1 if higher than wanted Temp+hyst, 0 if wantedTemp, -1 lower
  resultTemp = tempAvg();

  /* Affichage de la tempï¿½rature */
  Serial.print(resultTemp);
  Serial.println("Celsius: ");
  // switch de la rï¿½sistance

  if (resultTemp < wantedTemp - hysteresis)
  { digitalWrite(heaterPin, HIGH);
    //Serial.println("HIGH");
    return (-1);
  }
  else if (resultTemp > wantedTemp + hysteresis)
  { digitalWrite(heaterPin, LOW);
    //Serial.println("LOW");
    return (1);
  }

  return (0);
}

int tempMonitoringCool(float resultTemp, float wantedTemp)
{
  //control heating device accordingly to the measured temp
  //return 1 if higher than wanted Temp+hyst, 0 if wantedTemp, -1 lower
  resultTemp = tempAvg();


  /* Affichage de la tempï¿½rature */
  Serial.print(resultTemp);
  Serial.println("Celsius: ");

  // switch de la rï¿½sistance

  if (resultTemp < wantedTemp - hysteresis)
  { digitalWrite(coolerPin, LOW);
    //Serial.println("HIGH");
    return (-1);
  }
  else if (resultTemp > wantedTemp + hysteresis)
  { digitalWrite(coolerPin, HIGH);
    //Serial.println("LOW");
    return (1);
  }

  return (0);
}

int heatAmplification(float wantedTemp)
{
  //heat a Sample to the wanted temperature
  float resultTemp = 0;
  int tempTest;

  resultTemp = tempAvg();
  tempTest = tempMonitoringHeatAmpli(resultTemp, wantedTemp);

  while (tempTest == -1)
  {
    resultTemp = tempAvg();
    tempTest = tempMonitoringHeatAmpli(resultTemp, wantedTemp);
  }

  resultTemp = tempAvg();
  digitalWrite(heaterAmpli_Pin, LOW);
  Serial.println("Heating of the sample complete !");
  return (1);
}





int heatSample(float wantedTemp)
{
  //heat a Sample to the wanted temperature
  float resultTemp = 0;
  int tempTest;

  resultTemp = tempAvg();
  tempTest = tempMonitoringHeat(resultTemp, wantedTemp);

  while (tempTest == -1)
  {
    resultTemp = tempAvg();
    tempTest = tempMonitoringHeat(resultTemp, wantedTemp);
  }

  resultTemp = tempAvg();
  digitalWrite(heaterPin, LOW);
  Serial.println("Heating of the sample complete !");
  return (1);
}

int coolSample(float wantedTemp)
{
  //cool a Sample to the wanted temperature
  float resultTemp = 0;
  int tempTest;

  resultTemp = tempAvg();
  tempTest = tempMonitoringCool(resultTemp, wantedTemp);

  while (tempTest == 1)
  {
    resultTemp = tempAvg();
    tempTest = tempMonitoringCool(resultTemp, wantedTemp);
  }

  resultTemp = tempAvg();
  //digitalWrite(coolerPin, LOW); => we want to stay at ~10 Â°C
  Serial.println("cooling of the sample complete !");
  return (1);
}





//  actuator fonctions


void SetStrokePerc(float strokePercentage)
{
  if ( strokePercentage >= 1.0 && strokePercentage <= 99.0 )
  {
    int usec = 1000 + strokePercentage * ( 2000 - 1000 ) / 100.0 ;
    myServo.writeMicroseconds( usec );
  }
}

void SetStrokeMM(int strokeReq, int strokeMax)
{
  SetStrokePerc( ((float)strokeReq) / strokeMax );
}


